## 1. ################################################
# Intro to classification:
# The challenger disaster.

challenger = read.table("challenger.dat", head = T)
attach(challenger)
Temperature
Damage = (NFailed > 0)
Dp = as.numeric(Damage)

# making a nicer plot
Dp[14] = Dp[14]+0.05; Dp[17] = Dp[17]+0.1;
Dp[23] = Dp[23]+0.05; Dp[13] = Dp[13]-0.05

plot(Temperature, Dp, xlab = "Temperature", ylab = "Damage",
     col = c("red", "blue")[factor(Damage)])

lgstmodel = glm(Damage ~ Temperature, family = binomial(link = "logit"))
summary(lgstmodel)
phat = fitted(lgstmodel)
ghat = phat > 0.5

plot(Temperature, phat, xlab = "Temperature", ylab = "fitted p",
     col = c("red", "blue")[factor(Damage)])
abline(h = 0.5) # Bayes classification rule in two-class case

# Confusion matrix: slide 3.8
ConfMat = table("Actual" = Damage, "Predicted" = ghat)
TPR = ConfMat[2, 2] / (ConfMat[2, 2] + ConfMat[2, 1]) # sensitivity (focus on TRUE)
FPR = ConfMat[1, 2] / (ConfMat[1, 2] + ConfMat[1, 1]) 
Spec = 1 - FPR # specificity (focus on FALSE)
err = 1/length(Damage) * (ConfMat[1, 2] + ConfMat[2, 1]) # misclassification rate

## 2. ################################################
# A simulated example on classification:
# Linear discriminant analysis.

m0 = 80; m1 = 70;  # group means
s0 = 2; s1 = 2;    # standard deviations
# p0 and p1 represent prior probabilities of belonging to one
# class (G = 0) or the other (G = 1). We estimate them with
# sample frequencies (p_i = # (Y | G = i) / # Y, where #A is the 
# cardinality of set A. 
p0 = 0.5; p1 = 1 - p0 
xval = seq(65, 85, length = 100) # set range of x values for plotting

# We specify the prior densities according to our parametric
# assumption of normality: X|G_i ~ N(mu_i, sigma_i)
fx_0 = dnorm(xval, mean = m0, sd = s0); # Gaussian density f(x) for group 0
fx_1 = dnorm(xval, mean = m1, sd = s1);  # Gaussian density f(x) for group 1


# The posterior density (according to which we classify)
# is proportional to pi_i * f_i(x) 
p0 * dnorm(78, mean = m0, sd = s0) 
p1 * dnorm(78, mean = m1, sd = s1) 

# Plotting densities and discriminant functions (evaluated above)
par(mfcol = c(2, 1), mar = c(4, 4, 1.0, 0.5), oma = c(1.5, 2, 1, 1)) # graphic settings
plot(xval, fx_0, type = "l", col="red",    main = " Densities ",
     xlab = "",  ylab = "")   # densities
lines(xval, fx_1, col="blue", lwd=1, lty = 1)
plot(xval, log(p0) + log(fx_0), col = "red",  type = "l", main = " Discriminant function ",
     xlab = "",  ylab = "")  
lines(xval, log(p1) + log(fx_1), col="blue",  lty=1) # discriminant functions
legend("bottom", legend = c("G = 0", "G = 1"), col = c("red", "blue"), lty = 1)

# What happens if I change the unconditional probability
# of Y belonging to the group G = 0, pi_0? TRY!

# Misclassification probability in the LDA
p1g0 =  pnorm((75 - m0)/s0) # P(x < 75|G = 0)
p0g1 =  1 - pnorm((75 - m1)/s1) # P(x > 75|G = 1)
p1g0 + p0g1

# Now we turn to a simulation experiment in which we 
# classify observations from a population with the parameters 
# defined above, according to the previous discriminant rule.
N = 1000
N0 = round(p0 * N); N1 = N-N0 
x0 = rnorm(N0, m0, 1)
x1 = rnorm(N1, m1, s1)
G = c(rep(0, N0), rep(1, N1))
x = sort(c(x0, x1))
sigma = sd(c(x0, x1))
fxe_0 = dnorm(x, mean = m0, sd = sigma)   # Gaussian density f(x) for group 0
fxe_1 = dnorm(x, mean = m1, sd = sigma)   # Gaussian density f(x) for group 1
Ghat = (p1 * fxe_1 > p0 * fxe_0)  # Classifier
table(G,Ghat)  # Confusion matrix

# Plotting the i) joint density, (ii) prior densities (notice equal variance in LDA),
# and (iii) discrimination rule (affected by prior pi_0).
par(mfcol = c(3, 1), mar = c(4, 4, 1.0, 0.5), oma = c(1.5, 2, 1, 1)) # graphic settings
hist(x, 30)   # sample distribution of X (histogram)
plot(x, fxe_0, type = "l", col="red",    main = " Prior densities ",
     xlab = "",  ylab = "")   # notice that the LDA prior densities have equal variance
lines(x, fxe_1, col="blue", lwd=1, lty = 1) 
plot(x, log(p0) + log(fxe_0), col = "red",  type = "l", main = " Discriminant function ",
     xlab = "",  ylab = "")  
lines(x, log(p1) + log(fxe_1), col="blue",  lty=1) # discriminant functions
legend("bottom", legend = c("G = 0", "G = 1"), col = c("red", "blue"), lty = 1)
par(mfcol = c(1, 1))