## 1. ################################################
# Obtaining the coefficients in a logistic regression:
# Maximum likelihood estimation (explainer)

## Simple binomial case (slides 24, 25 with constant p)
prob = 0.1  # true Value
n = 100     # number of observations
vY = rbinom(n, 1, prob) # training sample 
vY
table(vY)

vP = seq(0.001, 0.999, 0.001)
vLoglik = sum(vY) * log(vP) + (n - sum(vY)) * log(1 - vP) # binomial log-likelihood

# The next two plots show the log-likelihood (the likelihood respectively)
# of our model for varying values of p given the observations y. 
par(mfrow = c(1,2))
plot(vP, vLoglik, type = 'l', col = "red", main = "Log-likelihood") 
plot(vP, exp(vLoglik), type = 'l', col = "blue", main = "Likelihood")

# The likelihood (equivalently, loglikelihood) is maximum around the true value 
# of p (try increasing n). 
vP[vLoglik == max(vLoglik)] # maximum likelihood estimate

## Logistic model (slide 21):
# we now model p as a linear function of our covariates X. 
fLoglik <- function(beta_0, beta_1, x, y)
{
  dloglik = 0
  N = length(y)
  for (i in 1:N)
  {
    u_i = beta_0 + beta_1 * x[i]
    p_i = exp(u_i)/(1 + exp(u_i)) # t
    dloglik = dloglik + y[i] * log(p_i) + (1 - y[i])*log(1 - p_i)
  }
  return(dloglik)
}

# Given X and y (our training sample), we would like to find beta_0 and beta_1
# such that the log-likelihood function (floglik) is maximum.

challenger = read.table("challenger.dat",head=T)
attach(challenger)
Temperature;
Damage = (NFailed > 0)

fLoglik(0, .1, Temperature, Damage)
fLoglik(0, -.1, Temperature, Damage)
fLoglik(-1, -.1, Temperature, Damage)
fLoglik(1, -.1, Temperature, Damage)
fLoglik(15.3, -.24, Temperature, Damage) # the loglik increases


## 2. ################################################
# An application of logistic regression:
# German credit data

## Data import and overview
rm(list = ls()) # remove previous objects
tCdata = read.csv("german.csv", header = TRUE)
N = nrow(tCdata)
names(tCdata)
tCdata$Crunworthy = tCdata$Group - 1
attach(tCdata)

# Dependent variable: creditworthiness:
#  1 : credit-worthy;  2 : not credit-worthy 

table(Crunworthy)
table(Crunworthy, Purpose)

plot(x = CreditAmount, y = Crunworthy , xlab = "Credit Amount", ylab="Credit Unworthy",
     col = c("red", "blue")[factor(Crunworthy)]) 

## Model full estimation 
model_full_formula = 
  factor(Crunworthy) ~
  Duration + factor(StatusCAccount) + factor(CreditHistory) +
  factor(Purpose) + CreditAmount + factor(Purpose) +
  factor(SAccBonds) + factor(Employment) + factor(Instalment) +
  factor(PStatusSex) + factor(OtherDebtors) +
  factor(Residence) + factor(Property) + Age +
  factor(OtherPlans) + factor(Housing) + NCredits +
  factor(Job) + NPeople + factor(Telephone) + factor(Foreign) +
  I(CreditAmount ^ 2) + I(Duration ^ 2) + I(Duration * CreditAmount)

oMfull = glm(model_full_formula,  family = binomial(link = "logit"),
             data = tCdata)
summary(oMfull)

## Understanding the model
vRr = residuals(oMfull, type = "response")
vRp = residuals(oMfull, type = "pearson")
vRsp = rstandard(oMfull, type = "pearson") # standardized by the h index
hist(vRsp, 30, probability = TRUE)
curve(dnorm(x, mean = mean(vRsp), sd = sd(vRsp)), 
      col = "red", lwd = 2, add = TRUE) 

barplot(vRp)
outlier = which(abs(vRp) > quantile(r, 0.95))
oMfull$fitted.values[outlier]
oMfull$y[outlier]

vD = residuals(oMfull, type = "deviance")  # deviance residuals
hist(vD, 30, probability = TRUE)
curve(dnorm(x, mean = mean(vD), sd = sd(vD)), 
      col = "red", lwd = 2, add = TRUE) 
quantile(vD)
barplot(vD)
sum(vD^2); # this is equal to the deviance in the summary table

p.hat = predict(oMfull, type = "response") # predicted values
logits = predict(oMfull)        # returns the linear predictor X*b 
exp(logits)/(1 + exp(logits)) == p.hat
plot(x = logits, y = Crunworthy, col = c("red", "blue")[factor(Group)]);
lines(logits, p.hat, type = "p", col = c("red", "blue")[factor(Group)]) 

# Missclassification rate
vPredicted = (oMfull$fitted.values > 0.5)
table(vPredicted, Crunworthy)
err = (76+131)/N


## Model selection for prediction
set.seed(123)   
Ntr = 0.7 * N         
s = sample(1 : N, Ntr) 
tTrain = tCdata[ s,]   
tTest = tCdata[- s,]   
summary(tTrain)   # data set summary 
summary(tTest)

detach(tCdata)
attach(tTrain)

model_restr_formula = factor(Crunworthy) ~ Duration + CreditAmount
oMrestr = glm(model_restr_formula, family = binomial(link = "logit"))
summary(oMrestr)

# stepwise selection (using the base stat library)
oSel.f = step(object = oMrestr, scope = formula(oMfull), direction = "forward", k = log(N))
summary(oSel.f)
oSel.b = step(object = oMfull, scope = formula(oMfull), direction = "backward", k = log(N))
summary(oSel.b)
oMsel =  glm(formula(oSel.b), family = binomial(link = "logit"))
summary(oMsel)

vPredFull = predict(object = oMfull, newdata = tTest, type = "response") > 0.5
vPredSel = predict(object = oMsel, newdata = tTest, type = "response") > 0.5

tPredFull = table(tTest$Crunworthy, vPredFull)
tPredSel = table(tTest$Crunworthy, vPredSel)

tp.full = tPredFull[2, 2] / (tPredFull[2, 1] + tPredFull[2, 2])
tp.sel = tPredSel[2, 2] / (tPredSel[2, 1] + tPredSel[2, 2])

err.full = (tPredFull[1, 2] + tPredFull[2, 1]) / nrow(tTest)
err.sel = (tPredSel[1, 2] + tPredSel[2, 1]) / nrow(tTest)